#-*-coding:utf-8*-*

from os.path import join
from json import load
from py.test import raises
from creole.loader import config_save_values
from tiramisu.error import PropertiesOptionError
from eolegenconfig import lib
ID_ = 'test'
DICO_DIR = '/usr/share/creole/tests/dicos'
CONFIG_DIR = '/usr/share/creole/tests/configs'

def _calc_config_file(test):
    """
    nom du fichier eol lié au test
    """
    return join(CONFIG_DIR, '{0}.eol'.format(test))

def _load(test, erase=True):
    """
    Chargement des dictionnaires liés au test
    Initialisation du fichier eol lié au test
    """
    config_file = _calc_config_file(test)
    if erase:
        file(config_file, 'w')
    return lib.get_config(ID_, force_dirs=join(DICO_DIR, test),
                          force_configfile=config_file)

def _reload(test):
    """
    recharge une configuration enregistrée
    """
    _load(test, erase=False)

def _save(test):
    """
    Enregistement des données
    """
    config_file = _calc_config_file(test)
    config = lib.get_config(ID_)
    config_save_values(config, 'creole', eol_file=config_file)

def test_basic_rules():
    """
    Vérifie l'application de certines règles de base #7432
    """
    config = _load('basic')
    # variable obligatoire sans valeur par défaut => mode basique
    assert lib.get_variable(ID_, 'test', 'obligatoire')['mode'] == 'basic'
    # variable non obligatoire avec valeur par défaut => obligatoire
    assert lib.get_variable(ID_, 'test', 'facultatif')['mandatory']

def test_calc_multi_condition():
    """
    calc_multi_condition avec une variable désactivée #8439
    calc_multi_condition avec 2 valeurs différentes en entrée #9797
    """
    config = _load('multicondition')
    assert config.creole.test.target1 == 'resultat2'
    assert config.creole.test.target2 == 'resultat1'
    assert config.creole.test.target3 == 'oui'
    lib.set_value(ID_, 'test', 'condition', 'non')
    assert config.creole.test.target1 == 'resultat1'
    assert config.creole.test.target2 == 'mismatch'
    assert config.creole.test.target3 == 'non'

def test_auto_save():
    """
    auto_save sur une variable multi (#8020)
    """
    config = _load('autosave')
    assert config.creole.test.autosave == [u'a', u'b']
    _save('autosave')
    content = load(file(_calc_config_file('autosave')))
    assert content['autosave'] == {'owner': 'gen_config', 'val': [u'a', u'b']}

def test_auto_freeze():
    """
    Comportement des variables auto_freeze='True'
    """
    config = _load('autofreeze')
    lib.set_mode(ID_, 'expert')
    # variable auto_freeze => mode basic
    assert lib.get_variable(ID_, 'test', 'autofreeze')['mode'] == 'basic'
    # mais exception si mode='expert' (#7349)
    assert lib.get_variable(ID_, 'test', 'autofreeze_x')['mode'] == 'expert'
    lib.set_value(ID_, 'test', 'autofreeze', 'freeze')
    _save('autofreeze')
    content = load(file(_calc_config_file('autofreeze')))
    # les variables auto_freeze sont enregistrées
    assert content[u'autofreeze'] == {u'owner': u'gen_config', u'val': u'freeze'}
    assert content[u'autofreeze_x'] == {u'owner': u'gen_config', u'val': None}
    config = _reload('autofreeze')
    # passage en mode debug pour accéder à la variable hidden "module_instancie"
    lib.set_debug(ID_, True)
    if lib.get_variable(ID_, 'general', 'module_instancie')['value'] == 'oui':
        # la variable n'est plus éditable après instance
        assert lib.get_variable(ID_, 'test', 'autofreeze')['editable'] == False
    else:
        # la variable est toujours éditable avant instance
        assert lib.get_variable(ID_, 'test', 'autofreeze')['editable'] == True
    # mais exception pour None (FIXME)
    assert lib.get_variable(ID_, 'test', 'autofreeze_x')['editable']

def test_wrong_calc():
    """
    pas de traceback en cas de valeur calculée invalide
    """
    config = _load('wrong_calc')
    assert config.creole.test.test_value == 'FR'
    assert config.creole.test.test_country_name == 'FR'
    assert config.creole.test.test_country_name2 == 'FR'
    lib.set_value(ID_, 'test', 'test_value', 'EU')
    assert config.creole.test.test_country_name == 'EU'
    assert config.creole.test.test_country_name2 == 'EU'
    lib.set_value(ID_, 'test', 'test_value', 'I2')
    var = lib.get_variable(ID_, 'test', 'test_country_name')
    assert var['value'] is None
    assert 'test_country_name' in var.get('warning', '')
    assert config.creole.test.test_country_name2 == 'FR'

def test_redefine():
    """
    Vérifie que redefine et remove_check font leur travail #9468
    Vérifie le redefine entre fill et auto #10475
    Vérifie que redefine et remove_condition font leur travail #13729
    """
    config = _load('redefine')
    # tests redefine et remove_check
    assert config.creole.test.abc1 == 'a'
    assert config.creole.test.abc2 == 'b'
    assert config.creole.test.abc3 == 'b'
    raises(ValueError, "lib.set_value(ID_, 'test', 'abc1', 'd')")
    raises(ValueError, "lib.set_value(ID_, 'test', 'abc2', 'd')")
    lib.set_value(ID_, 'test', 'abc3', 'd')
    assert config.creole.test.abc3 == 'd'
    # tests redefine sur fill et auto
    assert config.creole.test.fill2auto == 'valeur1valeur2'
    assert config.creole.test.auto2fill == 'valeur'
    raises(PropertiesOptionError, "lib.set_value(ID_, 'test', 'fill2auto', 'valeur3')")
    lib.set_value(ID_, 'test', 'auto2fill', 'valeur3')
    assert config.creole.test.auto2fill == 'valeur3'
    # tests redefine et remove_condition
    raises(PropertiesOptionError, "lib.set_value(ID_, 'test', 'disabled_if', 'val')")
    lib.set_value(ID_, 'test', 'disabled_if_redefine', 'val')
    lib.set_value(ID_, 'test', 'enabled_if', 'val')
    raises(PropertiesOptionError, "lib.set_value(ID_, 'test', 'enabled_if_redefine', 'val')")
    raises(PropertiesOptionError, "lib.set_value(ID_, 'test', 'frozen_if', 'val')")
    lib.set_value(ID_, 'test', 'frozen_if_redefine', 'val')

def test_check():
    """
    Vérifie le fonctionnement de optional & hidden
    sur les check avec des variables #9472
    """
    config = _load('check')
    # variable inconnue dans le check
    lib.set_value(ID_, 'test', 'check_unknown', 'ok')
    assert config.creole.test.check_unknown == 'ok'
    # variable désactivable dans le check
    lib.set_value(ID_, 'test', 'check_disabled', 'ok')
    assert config.creole.test.check_disabled == 'ok'
    raises(ValueError, "lib.set_value(ID_, 'test', 'check_disabled', 'sample')")
    lib.set_value(ID_, 'test', 'disable', 'oui')
    lib.set_value(ID_, 'test', 'check_disabled', 'sample')
    assert config.creole.test.check_disabled == 'sample'
    lib.set_value(ID_, 'test', 'disable', 'non')
    raises(ValueError, "lib.set_value(ID_, 'test', 'check_disabled', 'sample')")
    lib.set_value(ID_, 'test', 'check_disabled', 'ok')
    assert config.creole.test.check_disabled == 'ok'

def test_calc_multi_val():
    """
    Vérifie le fonctionnement de calc_multi_val
    Ajouté pour valider #10251 mais pourra être complété
    """
    config = _load('multival')
    # variable non multi => None
    assert config.creole.test.mail_src == None
    # variable multi  => []
    assert config.creole.test.mail_dest == []
    # désactivation de "mail_dest"
    lib.set_value(ID_, 'test', 'condition', 'non')
    # adresse invalide en source
    lib.set_value(ID_, 'test', 'mail_src', 'badmail')
    raises(PropertiesOptionError, "config.creole.test.mail_dest")
    # ré-activation de "mail_dest"
    lib.set_value(ID_, 'test', 'condition', 'oui')
    raises(ValueError, "config.creole.test.mail_dest")
